#!/bin/bash -e
#
# Copyright 2016 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

ARCH=$(dpkg --print-architecture)

argv0="$0"
opttemp=$(getopt -o f --longoptions 'force' --name "$argv0" -- "$@")
eval set -- "$opttemp"
unset opttemp

usage() {
	echo >&2 "usage: $argv0 <image-ucs-version> <role>"
	exit 1
}

FORCE=false
while [ $# -gt 0 ]; do
	case "$1" in
		-f|--force) FORCE=true; shift;;
		--) shift; break;;
		*) echo "Internal error!" ; exit 1 ;;
	esac
done

if [ $# -ne 2 ]; then
	usage
fi

IMAGE_UCS_VERSION="$1"
role="$2"
case "$role" in
	master|backup|slave|member|generic)
		base_image_basename="univention/ucs-stem-${ARCH}"
                ;;
        *)
                echo "unsupported system role"
		exit 1
		;;
esac

base_image_basename="univention/ucs-minbase-${ARCH}"
resultimage_name="univention/ucs-${role}-${ARCH}:${IMAGE_UCS_VERSION}"

image_name_is_available() {
	local image_name
	local image_uuid
	image_name="$1"
	image_uuid=$(docker inspect --format='{{.Id}}' "$image_name" 2>/dev/null || true)
	if [ -n "$image_uuid" ] && [ "$FORCE" = 'true' ]; then
		docker rmi "$image_uuid" || exit 1
		unset image_uuid
	fi
	if [ -n "$image_uuid" ]; then
		echo -e "ERROR: Docker image name $image_name already exists. UUID: $image_uuid"
		echo -e "ERROR: That name cannot be assigned to the new image."
		echo -e "INFO:  Consider assigning an alternative tag to the existing image:"
		echo -e "INFO:  \tdocker tag $image_name othername1"
		echo -e "INFO:  After that you may free the name my running:"
		echo -e "INFO:  \tdocker rmi $image_name"
		return 1
	fi
	return 0
}

if ! image_name_is_available "$resultimage_name"; then
	exit 1
fi

temp_dockerfile_dir="$(mktemp -d ${TMPDIR:-/var/tmp}/docker-build.XXXXXXXXXX)"
cleanup () {
	echo "Removing temporary $temp_dockerfile_dir"
	rm -rf "$temp_dockerfile_dir"
}
trap cleanup EXIT

cp -r "/usr/share/univention-docker-dev/build/${role}/"* "$temp_dockerfile_dir"

## If the Dockerfile has a "IMAGE_UCS_VERSION" dependency, replace it with the IMAGE_UCS_VERSION one:
# sed -i "s|^FROM ${base_image_basename}:IMAGE_UCS_VERSION$|FROM ${base_image_basename}:${IMAGE_UCS_VERSION}|" "$temp_dockerfile_dir/Dockerfile"
export IMAGE_UCS_VERSION
envsubst <"$temp_dockerfile_dir/Dockerfile.template" >"$temp_dockerfile_dir/Dockerfile"
cat "$temp_dockerfile_dir/Dockerfile"

## Start logging
exec > >(tee "create-docker-ucs-${role}-image.log") 2>&1
grep ^FROM "$temp_dockerfile_dir/Dockerfile"

## Start actual build
time docker build \
	-t "$resultimage_name" "$temp_dockerfile_dir"
if [ $? = 0 ]; then
	echo "Finished building $resultimage_name"
else
	trap - EXIT
fi 
